/*
 * adb.c
 *
 * Functions for managing author databases.
 *
 *       Created: 18th January, 1997
 * Last modified: 18th January, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include <stdio.h>
#include <gdbm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "authors.h"


const char szAuthorExt[] = "*.adb";


void AuthorNullifyDB(AUTHORDB *padb)
/*
 * Set an author database structure to be empty.
 *
 * AUTHORDB *padb	- the database structure to be nullified
 */
{
	padb->dbfInfo = NULL;
	padb->dbfDesc = NULL;
}


int AuthorOpenDB(char *pszFileName, AUTHORDB *padb)
/*
 * Open an author database for use; if it doesn't exit, create it.
 *
 * char *pszFileName	- the file name of the database to open (no extension)
 * AUTHORDB *padb	- the database structure to point to the file
 *
 * Returns:		- 1 on success
 *			  0 on failure
 */
{
	char szInfoFile[200], szDescFile[200];

	sprintf(szInfoFile, "%s.adb", pszFileName);
	sprintf(szDescFile, "%s.ddb", pszFileName);

	if ((padb->dbfInfo = gdbm_open(szInfoFile, 512, GDBM_WRCREAT, S_IREAD | S_IWRITE, NULL)) == NULL) {
		AuthorNullifyDB(padb);
		return (0);
	}
	if ((padb->dbfDesc = gdbm_open(szDescFile, 512, GDBM_WRCREAT, S_IREAD | S_IWRITE, NULL)) == NULL) {
		gdbm_close(padb->dbfInfo);
    		AuthorNullifyDB(padb);
		return (0);
	}

	return (1);
}


void AuthorReorganiseDB(AUTHORDB *padb)
/*
 * Re-organise an author database.
 *
 * AUTHORDB *padb	- the database structure to re-organise
 */
{
	gdbm_reorganize(padb->dbfInfo);
	gdbm_reorganize(padb->dbfDesc);
}


void AuthorCloseDB(AUTHORDB *padb)
/*
 * Close an author database.
 *
 * AUTHORDB *padb	- the database structure to close
 */
{
	gdbm_close(padb->dbfInfo);
	gdbm_close(padb->dbfDesc);
	AuthorNullifyDB(padb);
}
