/*
 * athreads.c
 *
 * Threads for use with author databases.
 *
 *       Created: 27th January, 1997
 * Last modified: 19th March, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include "quotes.h"
#include "threads.h"
#include <stdlib.h>


void ThreadFindAllAuthors(THREADINFO *pti)
/*
 * Start a thread to find all authors in the open database. 
 */
{
	HAB		hab;
	HMQ		hmq;
	AUTHORSEARCH	as;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (AuthorFullSearchInit(&(pti->pps->adb), &as)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
		while (AuthorFullSearchNext(&as))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindDescription(THREADINFO *pti)
/*
 * Start a thread to find all authors in the open database with a given string
 * in their description.
 */
{
	HAB		hab;
	HMQ		hmq;
	AUTHORSEARCH	as;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (AuthorDescSearchInit(&(pti->pps->adb), (char *)pti->pData, &as)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
		while (AuthorDescSearchNext(&as))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindAuthorName(THREADINFO *pti)
/*
 * Start a thread to find all authors in the open database with a given name.
 */
{
	HAB		hab;
	HMQ		hmq;
	AUTHORSEARCH	as;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (AuthorNameSearchInit(&(pti->pps->adb), (char *)pti->pData, &as)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
		while (AuthorNameSearchNext(&as))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(as.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}
