/*
 * general.c
 *
 * Miscellaneous functions.
 *
 *       Created: 18th January, 1997
 * Last modified: 9th April, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "general.h"


char *stripext(char *pszFileName)
/*
 * Strips the extension (if one exists) from a file name.
 *
 * char *pszFileName	- the file name to be stripped of its extension
 *
 * Returns:		- pszFileName
 */
{
	char *pchLastFS, *pchLastBS;

	pchLastFS = strrchr(pszFileName, '.');
	pchLastBS = strrchr(pszFileName, '\\');
	if (pchLastFS > pchLastBS)
		(*pchLastFS) = '\0';

	return (pszFileName);
}


char *strreplace(char *pszTemplate, char *pszMarker, char *pszReplace, char *pszOutput)
/*
 * Replace every occurrence of a marker in a string with a value.
 *
 * char *pszTemplate	- the template string containing markers
 * char *pszMarker	- the marker to be replaced
 * char *pszReplace	- the string to replace the marker with
 * char *pszOutput	- a string to hold the resulting string (output)
 *
 * Return:		- pszOutput
 */
{
	char *pcStart, *pcEnd;

	pszOutput[0] = '\0';
	pcStart = pszTemplate;
	while ((pcEnd = strstr(pcStart, pszMarker)) != NULL) {
		strncat(pszOutput, pcStart, pcEnd - pcStart);
		strcat(pszOutput, pszReplace);
		pcStart += strlen(pszMarker);
	}
	strcat(pszOutput, pcStart);

	return (pszOutput);
}


char *fcreates(char **ppszString, FILE *f)
/*
 * Place the contents of a file into a null-terminated string. Space for the string will
 * be allocated.
 *
 * char **ppszString	- will point to the buffer created by the function
 * FILE *f		- the file to be read into the buffer
 *
 * Returns:		- NULL if there is a memory allocation failure
 *			  pszString otherwise
 */
{
	char szLine[1000];
	long lPos;

	/* figure out how much space we need to allocate for the string */
	lPos = ftell(f);
	fseek(f, 0L, SEEK_END);
	if (((*ppszString) = (char *)malloc(ftell(f))) == NULL)
		return (NULL);
	fseek(f, lPos, SEEK_SET);

	(*ppszString)[0] = '\0';
	while (!feof(f)) {
		if (fgets(szLine, 1000, f) == NULL)
			break;
		strcat((*ppszString), szLine);
	}

	return (*ppszString);
}


int fexists(char *pszFileName)
/*
 * Check for the existence of a file.
 *
 * char *pszFileName	- the name of the file to check for
 *
 * Returns:		- 1 if the file exists
 *			  0 if it does not
 */
{
	FILE *f;
	int bRet;

	f = fopen(pszFileName, "r");
	bRet = f != NULL;
	if (bRet)
		fclose(f);

	return (bRet);
}
