/*
 * paint.c
 *
 * Window painting functions for the Quoteriser.
 *
 *       Created: 27th February, 1997
 * Last modified: 9th April, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#define INCL_GPI
#include <os2.h>
#include <stdio.h>
#include <string.h>
#ifdef QOTD
# include "qotdpm.h"
#else
# include "quoterpm.h"
#endif
#include "types.h"
#include "threads.h"
#include "html.h"


HAB habPaint;
HWND hwndPaint;


MRESULT EXPENTRY PaintWinProc(HWND, ULONG, MPARAM, MPARAM);
void TypesetString(HPS, char *, POINTL *, int);
void TypesetWord(HPS, char *, int *, int *, int, POINTL *);


void ThreadPaint(THREADINFO *pti)
/*
 * Thread used to draw the contents of the client window. It creates a
 * window procedure PaintWinProc() to handle calls from the main window
 */
{
	HMQ hmq;
	QMSG qmsg;

	habPaint = WinInitialize(0);
	hmq = WinCreateMsgQueue(habPaint, 0);
	WinRegisterClass(habPaint, "QPAINT", (PFNWP)PaintWinProc, 0, 0);

	/* create the paint window */
	hwndPaint = WinCreateWindow(HWND_OBJECT, "QPAINT", "", 0, 0, 0, 0, 0,
				    HWND_OBJECT, HWND_BOTTOM, 0, pti, NULL);

	/* go */
	while (WinGetMsg(habPaint, &qmsg, (HWND)NULL, 0, 0))
		WinDispatchMsg(habPaint, &qmsg);

	/* clean up */
	WinDestroyWindow(hwndPaint);
	WinDestroyMsgQueue(hmq);
	WinTerminate(habPaint);
}


MRESULT EXPENTRY PaintWinProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the paint thread. It re-paints the screen every time it
 * is called with a QM_PAINT message.
 */
{
	static THREADINFO	*pti;
	static HPS		hps;
	FONTMETRICS		fm;
	FATTRS			fattrs;
	RECTL			rectl;
	POINTL			ptPos;
	BOOL			bPrintAuthor, bPrintSource;
	char			szString[210];

	switch (message) {
		case WM_CREATE:
			pti = (THREADINFO *)PVOIDFROMMP(mp1);
			hps = WinGetPS(pti->hwndCaller);

			return (FALSE);

		case QM_PAINT:
			hps = WinBeginPaint(pti->hwndCaller, hps, NULL);
			GpiResetPS(hps, GRES_ALL);

			/* create our fonts */
			GpiQueryFontMetrics(hps, sizeof(FONTMETRICS), &fm);
			fattrs.usRecordLength = sizeof(FATTRS);
			fattrs.fsSelection = 0;
			fattrs.lMatch = 0;
			strcpy(fattrs.szFacename, fm.szFacename);
			fattrs.idRegistry = 0;
			fattrs.usCodePage = fm.usCodePage;
			fattrs.lMaxBaselineExt = fm.lMaxBaselineExt;
			fattrs.lAveCharWidth = fm.lAveCharWidth;
			fattrs.fsType = 0;
			fattrs.fsFontUse = 0;
			GpiCreateLogFont(hps, NULL, IDF_NORMAL, &fattrs);
			fattrs.fsSelection = FATTR_SEL_ITALIC;
			GpiCreateLogFont(hps, NULL, IDF_ITALIC, &fattrs);
			fattrs.fsSelection = FATTR_SEL_BOLD;
			GpiCreateLogFont(hps, NULL, IDF_BOLD, &fattrs);
			fattrs.fsSelection = FATTR_SEL_BOLD | FATTR_SEL_ITALIC;
			GpiCreateLogFont(hps, NULL, IDF_BOLDITALIC, &fattrs);

			GpiErase(hps);
			WinQueryWindowRect(pti->hwndCaller, &rectl);
			ptPos.x = 0;
			ptPos.y = rectl.yTop - fm.lMaxBaselineExt;
			switch (pti->pps->iMode) {
				case Q_SEARCHING:
					WinLoadString(habPaint, 0, IDS_SEARCHING, sizeof(szString), szString);
					TypesetString(hps, szString, &ptPos, rectl.xRight);
					break;

				case Q_QUOTE:
					if (pti->pps->pszText != NULL)
						TypesetString(hps, pti->pps->pszText, &ptPos, rectl.xRight);
					bPrintAuthor = strlen(pti->pps->szAuthorCode) > 0;
					bPrintSource = strlen(pti->pps->pqi->szSource) > 0;
					if (bPrintAuthor || bPrintSource)  {
						strcpy(szString, "<p>- ");
						if (bPrintAuthor) {
							strcat(szString, pti->pps->pai->szGivenName);
							if ((strlen(pti->pps->pai->szSurname) > 0) &&
							    (strlen(pti->pps->pai->szGivenName) > 0))
								strcat(szString, " ");
							strcat(szString, pti->pps->pai->szSurname);
						}
						if (bPrintAuthor && bPrintSource)
							strcat(szString, ", ");
						if (bPrintSource) {
							strcat(szString, "<i>");
							strcat(szString, pti->pps->pqi->szSource);
							strcat(szString, "</i>");
						}
						TypesetString(hps, szString, &ptPos, rectl.xRight);
					}
					break;

				case Q_AUTHOR:
					strcpy(szString, "<b>");
					strcat(szString, pti->pps->pai->szGivenName);
					if ((strlen(pti->pps->pai->szSurname) > 0) &&
					    (strlen(pti->pps->pai->szGivenName) > 0))
						strcat(szString, " ");
					strcat(szString, pti->pps->pai->szSurname);
					strcat(szString, "</b> (");
					strcat(szString, pti->pps->pai->szBirthYear);
					strcat(szString, " - ");
					strcat(szString, pti->pps->pai->szDeathYear);
					strcat(szString, ")<br><br>");
					TypesetString(hps, szString, &ptPos, rectl.xRight);
					if (pti->pps->pszDesc != NULL)
						TypesetString(hps, pti->pps->pszDesc, &ptPos, rectl.xRight);
					break;
			}

			/* destroy our fonts */
			GpiSetCharSet(hps, LCID_DEFAULT);
			GpiDeleteSetId(hps, IDF_NORMAL);
			GpiDeleteSetId(hps, IDF_ITALIC);
			GpiDeleteSetId(hps, IDF_BOLD);
			GpiDeleteSetId(hps, IDF_BOLDITALIC);

			WinEndPaint(hps);
			return (FALSE);

		case WM_DESTROY:
			WinReleasePS(hps);
			return (FALSE);

		default:
			return (WinDefWindowProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


void TypesetString(HPS hps, char *pszString, POINTL *pptlPos, int iWidth)
/*
 * Typeset a string using some simple HTML and display it.
 *
 * HPS hps		- the handle to the presentation space on which to typeset
 * char *pszString	- the string to be typeset
 * POINTL *pptlPos	- starting position; end position is returned here
 * int iWidth		- the paragraph width
 */
{
	char		*pszCurrent, *pszNext;
	char		szChunk[30], szWord[30];
	int		aiFont[30], aiStart[30];
	int		iChunk, iMacro, iTag, i;
	long		lWordWidth;
	BOOL		bHTMLEnd, bGotWord;
	BOOL		bItalics, bBold;
	POINTL		aptlExtent[TXTBOX_COUNT];
	POINTL		ptlCursor;
	FONTMETRICS	fm;

	pszCurrent = pszString;
	bHTMLEnd = FALSE;
	bItalics = bBold = FALSE;
	GpiSetCharSet(hps, IDF_NORMAL);
	GpiQueryFontMetrics(hps, sizeof(FONTMETRICS), &fm);
	memcpy(&ptlCursor, pptlPos, sizeof(POINTL));
	i = 0;
	aiFont[0] = IDF_NORMAL;
	aiStart[0] = 0;
	while (!bHTMLEnd) {

		/* construct the next word we have to typeset */
		szWord[0] = '\0';
		bGotWord = FALSE;
		lWordWidth = 0;
		aiFont[0] = aiFont[i];
		i = 0;
		while (!bGotWord) {
			iTag = 0;
			iChunk = HTMLGetNextChunk(pszCurrent, szChunk, &pszNext);
			switch (iChunk) {
				case HTML_WORD_END:
					bGotWord = TRUE;
				case HTML_WORD_MID:
					strcat(szWord, szChunk);
					GpiQueryTextBox(hps, strlen(szChunk), szChunk, TXTBOX_COUNT, aptlExtent);
					lWordWidth += aptlExtent[4].x - aptlExtent[1].x;
					break;

				case HTML_TAG_END:
					bGotWord = TRUE;
				case HTML_TAG_MID:
					iTag = HTMLParseTag(szChunk);
					switch (iTag) {
						case HTML_PARAGRAPH:
						case HTML_LINEBREAK:
							bGotWord = TRUE;
							break;

						case HTML_ITALICS_START:
							bItalics = TRUE;
							GpiSetCharSet(hps, (bBold)? IDF_BOLDITALIC : IDF_ITALIC);
							aiFont[++i] = (bBold)? IDF_BOLDITALIC : IDF_ITALIC;
							aiStart[i] = strlen(szWord);
							break;

						case HTML_ITALICS_END:
							bItalics = FALSE;
							GpiSetCharSet(hps, (bBold)? IDF_BOLD : IDF_NORMAL);
							aiFont[++i] = (bBold)? IDF_BOLD : IDF_NORMAL;
							aiStart[i] = strlen(szWord);
							break;

						case HTML_BOLD_START:
							bBold = TRUE;
							GpiSetCharSet(hps, (bItalics)? IDF_BOLDITALIC : IDF_BOLD);
							aiFont[++i] = (bItalics)? IDF_BOLDITALIC : IDF_BOLD;
							aiStart[i] = strlen(szWord);
							break;

						case HTML_BOLD_END:
							bBold = FALSE;
							GpiSetCharSet(hps, (bItalics)? IDF_ITALIC : IDF_NORMAL);
							aiFont[++i] = (bItalics)? IDF_ITALIC : IDF_NORMAL;
							aiStart[i] = strlen(szWord);
							break;
					}
					break;

				case HTML_MACRO_END:
					bGotWord = TRUE;
				case HTML_MACRO_MID:
					bGotWord = TRUE;
					if ((iMacro = HTMLParseMacro(szChunk)) != -1)
						sprintf(szChunk, "%c", iMacro);
					strcat(szWord, szChunk);
					GpiQueryTextBox(hps, strlen(szChunk), szChunk, TXTBOX_COUNT, aptlExtent);
					lWordWidth += aptlExtent[4].x - aptlExtent[1].x;
					break;

				case HTML_END:
					bGotWord = TRUE;
					bHTMLEnd = TRUE;
					break;
			}
			pszCurrent = pszNext;
		}


		if (szWord[0] != '\0')
			if ((ptlCursor.x + lWordWidth) <= iWidth) {
				/* the word will fit on this line */
				TypesetWord(hps, szWord, aiFont, aiStart, i + 1, &ptlCursor);
				ptlCursor.x += lWordWidth + fm.lAveCharWidth;
			} else {
				/* the word needs to go on a new line */
				ptlCursor.x = 0;
				ptlCursor.y -= fm.lMaxBaselineExt;
				TypesetWord(hps, szWord, aiFont, aiStart, i + 1, &ptlCursor);
				ptlCursor.x += lWordWidth + fm.lAveCharWidth;
			}
		if ((iTag == HTML_PARAGRAPH) || (iTag == HTML_LINEBREAK)) {
			/* insert a new line */
			ptlCursor.x = 0;
			ptlCursor.y -= fm.lMaxBaselineExt;
			if (iTag == HTML_PARAGRAPH)
				ptlCursor.y -= fm.lMaxBaselineExt;
		}
	}

	/* copy the finishing position into ptlPos to be returned */
	memcpy(pptlPos, &ptlCursor, sizeof(POINTL));
}


void TypesetWord(HPS hps, char *pszWord, int *aiFont, int *aiStart, int n, POINTL *pptlPos)
/*
 * Print a single word.
 *
 * HPS hps		- the handle of the presentation space to print on
 * char *pszWord	- the word to print
 * int *aiFont		- the vector containing IDs of fonts to use
 * int *aiStart	- the vector containing the starting positions for fonts
 * int n		- the length of aiFont and aiStart
 * POINTL *pptlPos	- the point to print the word at
 */
{
	char	szBlock[30], *pszCursor;
	POINTL	aptlExtent[TXTBOX_COUNT];
	POINTL	ptlCursor;
	int	i, k;

	memcpy(&ptlCursor, pptlPos, sizeof(POINTL));
	pszCursor = pszWord;
	for (i = 0; i < n; i++)
		if (pszCursor[0] != '\0') {
			if (i == (n - 1)) {
				strcpy(szBlock, pszCursor);
				k = strlen(pszCursor);
			} else {
				if ((k = aiStart[i + 1] - aiStart[i]) > 0) {
					strncpy(szBlock, pszCursor, k);
					szBlock[k] = '\0';
					pszCursor += k;
				}
			}
			if (k > 0) {
				GpiSetCharSet(hps, aiFont[i]);
				GpiCharStringAt(hps, &ptlCursor, strlen(szBlock), szBlock);
				GpiQueryTextBox(hps, strlen(szBlock), szBlock, TXTBOX_COUNT, aptlExtent);
				memcpy(&ptlCursor, aptlExtent + 4, sizeof(POINTL));
			}
		}
}
