/*
 * qdb.c
 *
 * Functions for managing quote databases.
 *
 *       Created: 18th January, 1997
 * Last modified: 18th January, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include <stdio.h>
#include <gdbm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "quotes.h"


const char szQuoteExt[] = "*.qdb";

void QuoteNullifyDB(QUOTEDB *pqdb)
/*
 * Set an quote database structure to be empty.
 *
 * QUOTEDB *qdb	- the database structure to be nullified
 */
{
	pqdb->dbfInfo = NULL;
	pqdb->dbfText = NULL;
}


int QuoteOpenDB(char *pszFileName, QUOTEDB *pqdb)
/*
 * Open an quote database for use; if it doesn't exit, create it.
 *
 * char *pszFileName	- the file name of the database to open (no extension)
 * QUOTEDB *pqdb	- the database structure to point to the file
 *
 * Returns:		- 1 on success
 *			  0 on failure
 */
{
	char szInfoFile[200], szTextFile[200];

	sprintf(szInfoFile, "%s.qdb", pszFileName);
	sprintf(szTextFile, "%s.tdb", pszFileName);

	if ((pqdb->dbfInfo = gdbm_open(szInfoFile, 512, GDBM_WRCREAT, S_IREAD | S_IWRITE, NULL)) == NULL) {
		QuoteNullifyDB(pqdb);
		return (0);
	}
	if ((pqdb->dbfText = gdbm_open(szTextFile, 512, GDBM_WRCREAT, S_IREAD | S_IWRITE, NULL)) == NULL) {
		gdbm_close(pqdb->dbfInfo);
		QuoteNullifyDB(pqdb);
		return (0);
	}

	return (1);
}


void QuoteReorganiseDB(QUOTEDB *pqdb)
/*
 * Re-organise an quote database.
 *
 * QUOTEDB *pqdb	- the database structure to re-organise
 */
{
	gdbm_reorganize(pqdb->dbfInfo);
	gdbm_reorganize(pqdb->dbfText);
}


void QuoteCloseDB(QUOTEDB *pqdb)
/*
 * Close an quote database.
 *
 * QUOTEDB *pqdb	- the database structure to close
 */
{
	gdbm_close(pqdb->dbfInfo);
	gdbm_close(pqdb->dbfText);
	QuoteNullifyDB(pqdb);
}
