/*
 * qotd.c
 *
 * Main procedures for the Quoteriser's Quote-of-the-Day utility.
 *
 *       Created: 31st March, 1997
 * Last modified: 9th April, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#define INCL_WIN
#include <os2.h>
#include <stdlib.h>
#include <stdio.h>
#include "qotdpm.h"
#include "threads.h"
#include "types.h"
#include "general.h"

HAB		hab;
HWND		hwndFrame, hwndMain;
char		szAppName[64];
PROGSTATE	ps;


MRESULT EXPENTRY MainWinProc(HWND, ULONG, MPARAM, MPARAM);

int main(void)
{
   	HMQ	hmq;
	QMSG	qmsg;
	ULONG	flFrameFlags = FCF_TITLEBAR | FCF_SYSMENU | FCF_MENU | FCF_SIZEBORDER | FCF_MINMAX | FCF_TASKLIST | FCF_SHELLPOSITION;
	char	szMainClass[] = "QOTDMAIN";

	if ((hab = WinInitialize(0)) == 0)
		return (1);
	hmq = WinCreateMsgQueue(hab, 0);

	/* create the main window */
	WinRegisterClass(hab, szMainClass, (PFNWP)MainWinProc, CS_SIZEREDRAW, 0); 
	WinLoadString(hab, 0, IDS_APPNAME, sizeof(szAppName), szAppName);
	hwndFrame = WinCreateStdWindow(HWND_DESKTOP, WS_VISIBLE, &flFrameFlags, szMainClass,
					szAppName, 0L, NULLHANDLE, IDM_MAIN, &hwndMain);

	if (hwndFrame != NULLHANDLE) {
		while (WinGetMsg(hab, &qmsg, (HWND)NULL, 0, 0))
			WinDispatchMsg(hab, &qmsg);

		WinDestroyWindow(hwndFrame);
	}

	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	return (0);
}


MRESULT EXPENTRY MainWinProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * WIndow procedure for the main window.
 */
{
	HINI			hini;
	static THREADINFO	ti;
	char			szString[200], szMessage[200], szFileName[150];

	switch (message) {
		case WM_CREATE:
			ps.iMode = Q_SEARCHING;
			WinInvalidateRect(hwndMain, NULL, TRUE);
			if ((hini = PrfOpenProfile(hab, "QUOTER.INI")) == NULLHANDLE) {
				WinLoadString(hab, 0, IDS_INIFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				return ((MRESULT)TRUE);
			}
			PrfQueryProfileString(hini, "QofD", "QDB", "", szFileName, sizeof(szFileName));
			if (!fexists(szFileName)) {
				WinLoadString(hab, 0, IDS_NODB, sizeof(szString), szString);
				sprintf(szMessage, szString, szFileName);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				PrfCloseProfile(hini);
				return ((MRESULT)TRUE);
			}
			stripext(szFileName);
			if (!QuoteOpenDB(szFileName, &(ps.qdb))) {
				WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
				sprintf(szMessage, szString, szFileName);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				PrfCloseProfile(hini);
				return ((MRESULT)TRUE);
			}
			PrfQueryProfileString(hini, "QofD", "ADB", "", szFileName, sizeof(szFileName));
			if (szFileName[0] != '\0') {
				stripext(szFileName);
				if (!AuthorOpenDB(szFileName, &(ps.adb))) {
					WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
					sprintf(szMessage, szString, szFileName);
					WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				}
			} else
				AuthorNullifyDB(&(ps.adb));
			PrfCloseProfile(hini);
			ti.hwndCaller = hwnd;
			ti.pps = &ps;
			if (_beginthread((THREADPROC)ThreadPaint, NULL, THREADSTACK, (void *)(&ti)) == -1) {
				WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				return ((MRESULT)TRUE);
			}
			if (_beginthread((THREADPROC)ThreadFindRandomQuote, NULL, THREADSTACK, (void *)(&ti)) == -1) {
				WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				return ((MRESULT)TRUE);
			}
			return (FALSE);

		case WM_ERASEBACKGROUND:
			return ((MRESULT)TRUE);

		case WM_PRESPARAMCHANGED:
			WinInvalidateRect(hwndMain, NULL, TRUE);
			return (FALSE);

		case WM_PAINT:
			WinSendMsg(hwndPaint, QM_PAINT, NULL, NULL);
			return (FALSE);

		case WM_DESTROY:
			/* free all of our memory */
			QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
			AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
			QuoteCloseDB(&(ps.qdb));
			if (ps.adb.dbfInfo != NULL)
				AuthorCloseDB(&(ps.adb));
			return (FALSE);

		case QM_FOUND:
			if (PVOIDFROMMP(mp1) == NULL) {
				WinLoadString(hab, 0, IDS_EMPTYDB, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
			} else {
				strcpy(ps.szQuoteCode, (char *)PVOIDFROMMP(mp1));
				QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
				if ((ps.adb.dbfInfo != NULL) && (ps.pqi->szAuthorCode[0] != '\0')) {
					strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
					AuthorGetAuthor(&(ps.adb), ps.szAuthorCode, &(ps.pai), &(ps.pszDesc));
				}
				ps.iMode = Q_QUOTE;
				WinInvalidateRect(hwndMain, NULL, TRUE);
			}
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDM_EXIT:
					ps.iMode = Q_NONE;
					WinSendMsg(hwnd, WM_CLOSE, NULL, NULL);
					return (FALSE);
			}

		default:
			return (WinDefWindowProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}
