/*
 * quoter.c
 *
 * Main procedures for the Quoteriser (including window procedures for
 * the user interface).
 *
 *       Created: 16th January, 1997
 * Last modified: 8th April, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#define INCL_WIN
#include <os2.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "quoterpm.h"
#include "types.h"
#include "authors.h"
#include "quotes.h"
#include "general.h"
#include "threads.h"


/* messages to pass to EditQuoteBox and EditAuthorBox */
#define Q_ADD		0
#define Q_EDIT		1


HAB		hab;
HWND		hwndFrame, hwndMain, hwndMenu;
char  		szAppName[64];
PROFILE	prf;
PROGSTATE	ps;


MRESULT EXPENTRY MainWinProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY IntroBoxProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY AboutBoxProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY SettingsBoxProc(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY ChooseQuoteBox(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY EditQuoteBox(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY ChooseAuthorBox(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY EditAuthorBox(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY QuoteOfTheDayBox(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY GetTextBox(HWND, ULONG, MPARAM, MPARAM);
HAPP StartEditor(HWND, char *);
void GetINISettings(void);
void SetINISettings(void);


int main(void)
{
	HMQ	hmq;
	QMSG	qmsg;
	ULONG	flFrameFlags = FCF_TITLEBAR | FCF_SYSMENU | FCF_MENU | FCF_SIZEBORDER | FCF_MINMAX | FCF_TASKLIST | FCF_SHELLPOSITION;
	char	szMainClass[] = "MAIN";

	if ((hab = WinInitialize(0)) == 0)
		return (1);
	hmq = WinCreateMsgQueue(hab, 0);

	/* display an introductory dialogue box */
	WinDlgBox(HWND_DESKTOP, HWND_DESKTOP, IntroBoxProc, NULLHANDLE, IDD_INTRO, (PVOID)NULL);

	/* create the main window */
	WinRegisterClass(hab, szMainClass, (PFNWP)MainWinProc, CS_SIZEREDRAW, 0); 
	WinLoadString(hab, 0, IDS_APPNAME, sizeof(szAppName), szAppName);
	hwndFrame = WinCreateStdWindow(HWND_DESKTOP, WS_VISIBLE, &flFrameFlags, szMainClass,
					szAppName, 0L, NULLHANDLE, IDM_MAIN, &hwndMain);

	if (hwndFrame != NULLHANDLE) {
		hwndMenu = WinWindowFromID(hwndFrame, FID_MENU);

		/* initialise menu item enable states */
		WinEnableMenuItem(hwndMenu, IDM_REORG_AUTHORS, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_REORG_QUOTES, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_CLOSE_AUTHORS, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_CLOSE_QUOTES, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_AUTHORS, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_ADD_AUTHOR, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_EDIT_AUTHOR, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_ALL, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_NAME, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_DESC, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_DEL_AUTHOR, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_QUOTES, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_ADD_QUOTE, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_EDIT_QUOTE, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_ALL, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_KEY, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_TEXT, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_NAME, FALSE);
		WinEnableMenuItem(hwndMenu, IDM_DEL_QUOTE, FALSE);

		while (WinGetMsg(hab, &qmsg, (HWND)NULL, 0, 0))
			WinDispatchMsg(hab, &qmsg);

		QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
		AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));

		WinDestroyWindow(hwndFrame);
	}

	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	return (0);
}


MRESULT EXPENTRY MainWinProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the main window.
 */
{
	FILEDLG			fd;
	char			szString[200], szMessage[200];
	static HAPP		happEditor = NULLHANDLE;
	static THREADINFO	ti;
	SEARCHINFO		si;

	switch (message) {
		case WM_CREATE:
			GetINISettings();
			QuoteNullifyDB(&(ps.qdb));
			AuthorNullifyDB(&(ps.adb));
			happEditor = NULLHANDLE;
			ps.iMode = Q_NONE;
			ps.pqi = NULL;
			ps.pai = NULL;
			ps.pszText = ps.pszDesc = NULL;
			ti.hwndCaller = hwnd;
			ti.pps = &ps;
			if (_beginthread((THREADPROC)ThreadPaint, NULL, THREADSTACK, (void *)(&ti)) == -1) {
				WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				return ((MRESULT)TRUE);
			}
			return (FALSE);

		case WM_ERASEBACKGROUND:
			return ((MRESULT)TRUE);

		case WM_APPTERMINATENOTIFY:
			happEditor = NULLHANDLE;
			return (FALSE);

		case WM_PRESPARAMCHANGED:
			WinInvalidateRect(hwndMain, NULL, TRUE);
			return (FALSE);

		case WM_PAINT:
			WinSendMsg(hwndPaint, QM_PAINT, NULL, NULL);
			return (FALSE);

		case WM_DESTROY:
			/* free all of our memory */
			QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
			AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));

			/* ensure that the databases are closed before exiting */
			if (ps.qdb.dbfInfo != NULL)
				QuoteCloseDB(&(ps.qdb));
			if (ps.adb.dbfInfo != NULL)
				AuthorCloseDB(&(ps.adb));
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDM_OPEN_QUOTES:
					ps.iMode = Q_NONE;
					memset(&fd, 0, sizeof(FILEDLG));
					fd.cbSize = sizeof(FILEDLG);
					fd.fl = FDS_OPEN_DIALOG | FDS_CENTER;
					strcpy(fd.szFullFile, szQuoteExt);
					WinFileDlg(HWND_DESKTOP, hwndFrame, &fd);
					if (fd.lReturn == DID_OK) {
						stripext(fd.szFullFile);
						if (!QuoteOpenDB(fd.szFullFile, &(ps.qdb))) {
							WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
							sprintf(szMessage, szString, fd.szFullFile);
							WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
						} else {
							WinEnableMenuItem(hwndMenu, IDM_OPEN_QUOTES, FALSE);
							WinEnableMenuItem(hwndMenu, IDM_REORG_QUOTES, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_CLOSE_QUOTES, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_QUOTES, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_ADD_QUOTE, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_EDIT_QUOTE, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_ALL, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_KEY, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_TEXT, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_DEL_QUOTE, TRUE);
							if (ps.adb.dbfInfo != NULL)
								WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_NAME, TRUE);
						}
					}
					QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_OPEN_AUTHORS:
					ps.iMode = Q_NONE;
					memset(&fd, 0, sizeof(FILEDLG));
					fd.cbSize = sizeof(FILEDLG);
					fd.fl = FDS_OPEN_DIALOG | FDS_CENTER;
					strcpy(fd.szFullFile, szAuthorExt);
					WinFileDlg(HWND_DESKTOP, hwndFrame, &fd);
					if (fd.lReturn == DID_OK) {
						stripext(fd.szFullFile);
						if (!AuthorOpenDB(fd.szFullFile, &(ps.adb))) {
							WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
							sprintf(szMessage, szString, fd.szFullFile);
							WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
						} else {
							WinEnableMenuItem(hwndMenu, IDM_OPEN_AUTHORS, FALSE);
							WinEnableMenuItem(hwndMenu, IDM_REORG_AUTHORS, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_CLOSE_AUTHORS, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_AUTHORS, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_ADD_AUTHOR, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_EDIT_AUTHOR, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_ALL, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_NAME, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_DESC, TRUE);
							WinEnableMenuItem(hwndMenu, IDM_DEL_AUTHOR, TRUE);
							if (ps.qdb.dbfInfo != NULL)
								WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_NAME, TRUE);

						}
					}
					QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_REORG_QUOTES:
					if (_beginthread((THREADPROC)QuoteReorganiseDB, NULL, THREADSTACK, (void *)&(ps.qdb)) == -1) {
						WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
						WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					}
					return (FALSE);

				case IDM_REORG_AUTHORS:
					if (_beginthread((THREADPROC)AuthorReorganiseDB, NULL, THREADSTACK, (void *)&(ps.adb)) == 1) {
						WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
						WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					}
					return (FALSE);

				case IDM_CLOSE_QUOTES:
					ps.iMode = Q_NONE;
					QuoteCloseDB(&(ps.qdb));
					WinEnableMenuItem(hwndMenu, IDM_OPEN_QUOTES, TRUE);
					WinEnableMenuItem(hwndMenu, IDM_REORG_QUOTES, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_CLOSE_QUOTES, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_QUOTES, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_ADD_QUOTE, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_EDIT_QUOTE, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_ALL, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_KEY, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_TEXT, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_NAME, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_DEL_QUOTE, FALSE);
					QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_CLOSE_AUTHORS:
					ps.iMode = Q_NONE;
					AuthorCloseDB(&(ps.adb));
					WinEnableMenuItem(hwndMenu, IDM_OPEN_AUTHORS, TRUE);
					WinEnableMenuItem(hwndMenu, IDM_REORG_AUTHORS, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_CLOSE_AUTHORS, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_AUTHORS, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_ADD_AUTHOR, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_EDIT_AUTHOR, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_ALL, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_NAME, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_AUTHOR_DESC, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_GET_QUOTE_NAME, FALSE);
					WinEnableMenuItem(hwndMenu, IDM_DEL_AUTHOR, FALSE);
					AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_SETTINGS:
					WinDlgBox(HWND_DESKTOP, hwnd, SettingsBoxProc, NULLHANDLE, IDD_SETTINGS, NULL);
					return (FALSE);
 
				case IDM_ADD_QUOTE:
					ps.iMode = Q_NONE;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					WinDlgBox(HWND_DESKTOP, hwnd, EditQuoteBox, NULLHANDLE, IDD_EDITQUOTE, MPFROMLONG(Q_ADD));
					if (ps.pqi != NULL) {
						if (ps.adb.dbfInfo != NULL) {
							/* try to find the author in the current author database */
							if (AuthorGetAuthor(&(ps.adb), ps.pqi->szAuthorCode, &(ps.pai), &(ps.pszDesc)))
								strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
						}
						ps.iMode = Q_QUOTE;
					} else
						ps.iMode = Q_NONE;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_EDIT_QUOTE:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szQuoteCode[0] != '\0') {
						QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
						QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
						WinDlgBox(hwnd, hwnd, EditQuoteBox, NULLHANDLE, IDD_EDITQUOTE, MPFROMLONG(Q_EDIT));
						ps.szAuthorCode[0] = '\0';
						if (ps.adb.dbfInfo != NULL) {
							/* try to find the author in the current author database */
							if (AuthorGetAuthor(&(ps.adb), ps.pqi->szAuthorCode, &(ps.pai), &(ps.pszDesc)))
								strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
						}
						ps.iMode = Q_QUOTE;
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_DEL_QUOTE:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szQuoteCode[0] != '\0') {
						ps.szQuoteCode[0] = '\0';
						QuoteDeleteQuote(&(ps.qdb), ps.szQuoteCode);
						QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
						ps.iMode = Q_NONE;
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_GET_QUOTE_ALL:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szQuoteCode[0] != '\0') {
						QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
						ps.szAuthorCode[0] = '\0';
						QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
						if (ps.adb.dbfInfo != NULL) {
							/* try to find the author in the current author database */
							if (AuthorGetAuthor(&(ps.adb), ps.pqi->szAuthorCode, &(ps.pai), &(ps.pszDesc)))
								strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
							else
								ps.szAuthorCode[0] = '\0';
						}
						else
							ps.szAuthorCode[0] = '\0';
						ps.iMode = Q_QUOTE;
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_GET_QUOTE_KEY:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					szString[0] = '\0';
					WinDlgBox(HWND_DESKTOP, hwnd, GetTextBox, NULLHANDLE, IDD_GETTEXT, MPFROMP(szString));
					if (szString[0] != '\0') {
						si.iType = Q_KEY;
						si.pszString = szString;
						WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
						if (ps.szQuoteCode[0] != '\0') {
							QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
							AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
							ps.szAuthorCode[0] = '\0';
							QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
							if (ps.adb.dbfInfo != NULL) {
								/* try to find the author in the current author database */
								if (AuthorGetAuthor(&(ps.adb), ps.pqi->szAuthorCode, &(ps.pai), &(ps.pszDesc)))
									strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
							}
							ps.iMode = Q_QUOTE;
							WinInvalidateRect(hwndMain, NULL, TRUE);
						}
					}
					return (FALSE);

				case IDM_GET_QUOTE_TEXT:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					szString[0] = '\0';
					WinDlgBox(HWND_DESKTOP, hwnd, GetTextBox, NULLHANDLE, IDD_GETTEXT, MPFROMP(szString));
					if (szString[0] != '\0') {
						si.iType = Q_TEXT;
						si.pszString = szString;
						WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
						if (ps.szQuoteCode[0] != '\0') {
							QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
							AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
							ps.szAuthorCode[0] = '\0';
							QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
							if (ps.adb.dbfInfo != NULL) {
								/* try to find the author in the current author database */
								if (AuthorGetAuthor(&(ps.adb), ps.pqi->szAuthorCode, &(ps.pai), &(ps.pszDesc)))
									strcpy(ps.szAuthorCode, ps.pqi->szAuthorCode);
							}
							ps.iMode = Q_QUOTE;
							WinInvalidateRect(hwndMain, NULL, TRUE);
						}
					}
					return (FALSE);

				case IDM_GET_QUOTE_NAME:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szAuthorCode[0] != '\0') {
						si.iType = Q_NAME;
						si.pszString = ps.szAuthorCode;
						WinDlgBox(HWND_DESKTOP, hwnd, ChooseQuoteBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
						if (ps.szQuoteCode[0] != '\0') {
							QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
							QuoteGetQuote(&(ps.qdb), ps.szQuoteCode, &(ps.pqi), &(ps.pszText));
							AuthorGetAuthor(&(ps.adb), ps.szAuthorCode, &(ps.pai), &(ps.pszDesc));
							ps.iMode = Q_QUOTE;
							WinInvalidateRect(hwndMain, NULL, TRUE);
						} else
 							ps.szAuthorCode[0] = '\0';
					}
					return (FALSE);

				case IDM_ADD_AUTHOR:
					ps.iMode = Q_NONE;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					WinDlgBox(HWND_DESKTOP, hwnd, EditAuthorBox, NULLHANDLE, IDD_EDITAUTHOR, MPFROMLONG(Q_ADD));
					if (ps.pai != NULL)
						ps.iMode = Q_AUTHOR;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					return (FALSE);

				case IDM_EDIT_AUTHOR:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szAuthorCode[0] != '\0') {
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
						AuthorGetAuthor(&(ps.adb), ps.szAuthorCode, &(ps.pai), &(ps.pszDesc));
						WinDlgBox(hwnd, hwnd, EditAuthorBox, NULLHANDLE, IDD_EDITAUTHOR, MPFROMLONG(Q_EDIT));
						ps.iMode = Q_AUTHOR;
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_DEL_AUTHOR:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szAuthorCode[0] != '\0') {
						AuthorDeleteAuthor(&(ps.adb), ps.szAuthorCode);
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_GET_AUTHOR_ALL:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					si.iType = Q_FULL;
					si.pszString = szString;
					WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
					if (ps.szAuthorCode[0] != '\0') {
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
						AuthorGetAuthor(&(ps.adb), ps.szAuthorCode, &(ps.pai), &(ps.pszDesc));
						ps.iMode = Q_AUTHOR;
						WinInvalidateRect(hwndMain, NULL, TRUE);
					}
					return (FALSE);

				case IDM_GET_AUTHOR_DESC:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					WinDlgBox(HWND_DESKTOP, hwnd, GetTextBox, NULLHANDLE, IDD_GETTEXT, MPFROMP(szString));
					if (szString[0] != '\0') {
						si.iType = Q_TEXT;
						si.pszString = szString;
						WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
						if (ps.szAuthorCode[0] != '\0') {
							AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
							AuthorGetAuthor(&(ps.adb), ps.szQuoteCode, &(ps.pai), &(ps.pszDesc));
							ps.iMode = Q_AUTHOR;
							WinInvalidateRect(hwndMain, NULL, TRUE);
						}
					}
					return (FALSE);


				case IDM_GET_AUTHOR_NAME:
					ps.iMode = Q_SEARCHING;
					WinInvalidateRect(hwndMain, NULL, TRUE);
					ps.szQuoteCode[0] = '\0';
					ps.szAuthorCode[0] = '\0';
					szString[0] = '\0';
					WinDlgBox(HWND_DESKTOP, hwnd, GetTextBox, NULLHANDLE, IDD_GETTEXT, MPFROMP(szString));
					if (szString[0] != '\0') {
						si.iType = Q_NAME;
						si.pszString = szString;
						WinDlgBox(HWND_DESKTOP, hwnd, ChooseAuthorBox, NULLHANDLE, IDD_CHOOSECODE, MPFROMP(&si));
						if (ps.szAuthorCode[0] != '\0') {
							AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
							AuthorGetAuthor(&(ps.adb), ps.szQuoteCode, &(ps.pai), &(ps.pszDesc));
							ps.iMode = Q_AUTHOR;
							WinInvalidateRect(hwndMain, NULL, TRUE);
						}
					}
					return (FALSE);

				case IDM_QOTD:
					WinDlgBox(HWND_DESKTOP, hwnd, QuoteOfTheDayBox, NULLHANDLE, IDD_QOTD, NULL);
					return (FALSE);

				case IDM_EXIT:
					ps.iMode = Q_NONE;
					if (happEditor != NULLHANDLE)
						WinTerminateApp(happEditor);
					WinSendMsg(hwnd, WM_CLOSE, NULL, NULL);
					return (FALSE);

				case IDM_GENHELP:
					if (happEditor == NULLHANDLE)
						if ((happEditor = StartEditor(hwnd, "..\\doc\\quoter.txt")) == NULLHANDLE) {
							WinLoadString(hab, 0, IDS_SPAWNFAILED, sizeof(szString), szString);
							sprintf(szMessage, szString, prf.szEditor);
							WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
						}
					return (FALSE);

				case IDM_LICENSE:
					if (happEditor == NULLHANDLE)
						if ((happEditor = StartEditor(hwnd, "..\\doc\\COPYING")) == NULLHANDLE) {
							WinLoadString(hab, 0, IDS_SPAWNFAILED, sizeof(szString), szString);
							sprintf(szMessage, szString, prf.szEditor);
							WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
						}
					return (FALSE);

				case IDM_ABOUT:
					WinDlgBox(hwnd, hwnd, AboutBoxProc, NULLHANDLE, IDD_ABOUT, NULL);
					return (FALSE);
				}
			return (FALSE);

		default:
			return (WinDefWindowProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY IntroBoxProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the intro box. Does nothing except disappear when
 * either the timer is up, or the "Okay" button is pressed.
 */
{
	unsigned long	ulWidth, ulHeight;
	RECTL		rectl;

	switch (message) {
		case WM_INITDLG:
			WinStartTimer(hab, hwnd, IDT_INTRO, 5000);
			ulWidth = WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN);
			ulHeight = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);
			WinQueryWindowRect(hwnd, &rectl);
			WinSetWindowPos(hwnd, HWND_TOP, (ulWidth - rectl.xRight) >> 1, (ulHeight - rectl.yTop) >> 1,
					0, 0, SWP_MOVE | SWP_ACTIVATE);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_INTRO_OKAY:
					WinStopTimer(hab, hwnd, IDT_INTRO);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		case WM_TIMER:
			WinStopTimer(hab, hwnd, IDT_INTRO);
			WinDismissDlg(hwnd, TRUE);
			return (FALSE);

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY AboutBoxProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the about box. Does nothing except disappear when the
 * "Okay" button is pressed.
 */
{
	switch (message) {
		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_ABOUT_OKAY:
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY SettingsBoxProc(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the settings box. Used to set preferences for using the
 * Quoteriser.
 */
{
	switch (message) {
		case WM_INITDLG:
			WinSetDlgItemText(hwnd, IDC_SET_EDITOR, prf.szEditor);
			WinSetDlgItemText(hwnd, IDC_SET_EDPARAMS, prf.szEditorParams);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_SET_OKAY:
					WinQueryDlgItemText(hwnd, IDC_SET_EDITOR, sizeof(prf.szEditor), prf.szEditor);
					WinQueryDlgItemText(hwnd, IDC_SET_EDPARAMS, sizeof(prf.szEditorParams), prf.szEditorParams);
					SetINISettings();
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_SET_DEFAULT:
					WinSetDlgItemText(hwnd, IDC_SET_EDITOR, "E.EXE");
					WinSetDlgItemText(hwnd, IDC_SET_EDPARAMS, "%");
					return (FALSE);

				case IDC_SET_CANCEL:
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY EditQuoteBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the edit quote box. If mp2 is Q_ADD when the dialogue is
 * initialised, add the quote to the database. If mp2 is Q_EDIT, edit a quote
 * already existing in the database.
 */
{
	static char	szTempFileName[L_tmpnam];
	static char	szOrigCode[20];
	static int	iMode;
	static HAPP	happEditor = NULLHANDLE;
	char 		szString[100], szMessage[200];
	int		k, m, n;
	FILE		*f;

	switch (message) {
		case WM_INITDLG:
			happEditor = NULLHANDLE;
			tmpnam(szTempFileName);
			WinSendDlgItemMsg(hwnd, IDC_EDITQ_CODE, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.szQuoteCode) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITQ_SOURCE, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pqi->szSource) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITQ_ACODE, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pqi->szAuthorCode) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITQ_KEYWORDS, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pqi->aszKeyword) - 5), 0);
			iMode = (int)LONGFROMMP(mp2);
			if (iMode == Q_EDIT) {
				strcpy(szOrigCode, ps.szQuoteCode);
				WinSetDlgItemText(hwnd, IDC_EDITQ_CODE, ps.szQuoteCode);
				WinSetDlgItemText(hwnd, IDC_EDITQ_SOURCE, ps.pqi->szSource);
				WinSetDlgItemText(hwnd, IDC_EDITQ_ACODE, ps.pqi->szAuthorCode);
				szString[0] = '\0';
				k = 0;
				while ((ps.pqi->aszKeyword[k][0] != '\0') && (k < 5)) {
					strcat(szString, ps.pqi->aszKeyword[k]);
					strcat(szString, " ");
					k++;
				}
				WinSetDlgItemText(hwnd, IDC_EDITQ_KEYWORDS, szString);
				if ((f = fopen(szTempFileName, "w")) == NULL) {
					WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
					sprintf(szMessage, szString, szTempFileName);
					WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					WinDismissDlg(hwnd, TRUE);
				} else {
					if (ps.pszText != NULL)
						fputs(ps.pszText, f);
					fclose(f);
				}
			} else
				if ((ps.pqi = (QUOTEINFO *)malloc(sizeof(QUOTEINFO))) == NULL) {
					WinLoadString(hab, 0, IDS_NOMEM, sizeof(szMessage), szMessage);
					WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					WinDismissDlg(hwnd, TRUE);
				}
			return (FALSE);

		case WM_APPTERMINATENOTIFY:
			happEditor = NULLHANDLE;
			WinEnableControl(hwnd, IDC_EDITQ_TEXT, TRUE);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_EDITQ_TEXT:
					if ((happEditor = StartEditor(hwnd, szTempFileName)) != NULLHANDLE)
						WinEnableControl(hwnd, IDC_EDITQ_TEXT, FALSE);
					return (FALSE);

				case IDC_EDITQ_OKAY:
					WinQueryDlgItemText(hwnd, IDC_EDITQ_CODE, sizeof(ps.szQuoteCode), ps.szQuoteCode);
					if ((iMode == Q_EDIT) && (strcmp(ps.szQuoteCode, szOrigCode) != 0))
						QuoteDeleteQuote(&(ps.qdb), szOrigCode);
					WinQueryDlgItemText(hwnd, IDC_EDITQ_SOURCE, sizeof(ps.pqi->szSource), ps.pqi->szSource);
					WinQueryDlgItemText(hwnd, IDC_EDITQ_ACODE, sizeof(ps.pqi->szAuthorCode), ps.pqi->szAuthorCode);
					WinQueryDlgItemText(hwnd, IDC_EDITQ_KEYWORDS, sizeof(szString), szString);
					k = 0;
					n = 0;
					while ((sscanf(szString + n, "%s%n", ps.pqi->aszKeyword[k], &m) == 1) && (k < 5)) {
						if (strlen(ps.pqi->aszKeyword[k]) == 0)
							break;
						n += m;
						k++;
					}
					for (; k < 5; k++)
						ps.pqi->aszKeyword[k][0] = '\0';
					if ((f = fopen(szTempFileName, "r")) != NULL) {
						fcreates(&(ps.pszText), f);
						fclose(f);
					}
					DosDelete(szTempFileName);
					if (happEditor != NULLHANDLE)
						WinTerminateApp(happEditor);
					QuoteAddQuote(&(ps.qdb), ps.szQuoteCode, ps.pqi, ps.pszText);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_EDITQ_CANCEL:
					if (iMode == Q_ADD)
						QuoteFreeQuote(&(ps.pqi), &(ps.pszText));
					if (happEditor != NULLHANDLE)
						WinTerminateApp(happEditor);
					DosDelete(szTempFileName);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY EditAuthorBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the edit quote box. If mp2 is Q_ADD when the dialogue is
 * initialised, add the author to the database. If mp2 is Q_EDIT, edit an author
 * already existing in the database.
 */
{
	static char	szTempFileName[L_tmpnam];
	static char	szOrigCode[20];
	static int	iMode;
	static HAPP	happEditor = NULLHANDLE;
	char 		szString[100], szMessage[200];
	FILE		*f;

	switch (message) {
		case WM_INITDLG:
			happEditor = NULLHANDLE;
			tmpnam(szTempFileName);
			WinSendDlgItemMsg(hwnd, IDC_EDITA_CODE, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.szAuthorCode) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITA_SURNAME, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pai->szSurname) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITA_GNAME, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pai->szGivenName) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITA_BORN, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pai->szBirthYear) - 1), 0);
			WinSendDlgItemMsg(hwnd, IDC_EDITA_DIED, EM_SETTEXTLIMIT, MPFROMSHORT(sizeof(ps.pai->szDeathYear) - 1), 0);
			iMode = (int)LONGFROMMP(mp2);
			if (iMode == Q_EDIT) {
				strcpy(szOrigCode, ps.szAuthorCode);
				WinSetDlgItemText(hwnd, IDC_EDITA_CODE, ps.szAuthorCode);
				WinSetDlgItemText(hwnd, IDC_EDITA_SURNAME, ps.pai->szSurname);
				WinSetDlgItemText(hwnd, IDC_EDITA_GNAME, ps.pai->szGivenName);
				WinSetDlgItemText(hwnd, IDC_EDITA_BORN, ps.pai->szBirthYear);
				WinSetDlgItemText(hwnd, IDC_EDITA_DIED, ps.pai->szDeathYear);
				if ((f = fopen(szTempFileName, "w")) == NULL) {
					WinLoadString(hab, 0, IDS_OPENFAILED, sizeof(szString), szString);
					sprintf(szMessage, szString, szTempFileName);
					WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					WinDismissDlg(hwnd, TRUE);
				} else {
					if (ps.pszDesc != NULL)
						fputs(ps.pszDesc, f);
					fclose(f);
				}
			} else
				if ((ps.pai = (AUTHORINFO *)malloc(sizeof(AUTHORINFO))) == NULL) {
					WinLoadString(hab, 0, IDS_NOMEM, sizeof(szMessage), szMessage);
					WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
					WinDismissDlg(hwnd, TRUE);
				}
			return (FALSE);

		case WM_APPTERMINATENOTIFY:
			happEditor = NULLHANDLE;
			WinEnableControl(hwnd, IDC_EDITA_DESC, TRUE);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_EDITA_DESC:
					if ((happEditor = StartEditor(hwnd, szTempFileName)) != NULLHANDLE)
						WinEnableControl(hwnd, IDC_EDITA_DESC, FALSE);
					return (FALSE);

				case IDC_EDITA_OKAY:
					WinQueryDlgItemText(hwnd, IDC_EDITA_CODE, sizeof(ps.szAuthorCode), ps.szAuthorCode);
					if ((iMode == Q_EDIT) && (strcmp(ps.szAuthorCode, szOrigCode) != 0))
						AuthorDeleteAuthor(&(ps.adb), szOrigCode);
					WinQueryDlgItemText(hwnd, IDC_EDITA_SURNAME, sizeof(ps.pai->szSurname), ps.pai->szSurname);
					WinQueryDlgItemText(hwnd, IDC_EDITA_GNAME, sizeof(ps.pai->szGivenName), ps.pai->szGivenName);
					WinQueryDlgItemText(hwnd, IDC_EDITA_BORN, sizeof(ps.pai->szBirthYear), ps.pai->szBirthYear);
					WinQueryDlgItemText(hwnd, IDC_EDITA_DIED, sizeof(ps.pai->szDeathYear), ps.pai->szDeathYear);
					if ((f = fopen(szTempFileName, "r")) != NULL) {
						fcreates(&(ps.pszDesc), f);
						fclose(f);
						DosDelete(szTempFileName);
					}
					if (happEditor != NULLHANDLE)
						WinTerminateApp(happEditor);
					AuthorAddAuthor(&(ps.adb), ps.szAuthorCode, ps.pai, ps.pszDesc);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_EDITA_CANCEL:
					if (iMode == Q_ADD)
						AuthorFreeAuthor(&(ps.pai), &(ps.pszDesc));
					if (happEditor != NULLHANDLE)
						WinTerminateApp(happEditor);
					DosDelete(szTempFileName);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY ChooseQuoteBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the choose quote box. This allows the user to select a
 * quote from a list of codes. The codes to display are determined by the
 * SEARCHINFO structure pointed to by mp2 passed with the WM_INITDLG. Depending
 * on the iType field:
 *
 * Q_FULL	- all quotes
 * Q_KEY	- quotes with the keyword in the pszString field
 * Q_TEXT	- quotes containing the text in the pszString field
 * Q_NAME	- quotes from the author with the code in the pszString field
 */
{
	static THREADINFO	ti;
	static HWND		hwndList;
	char			szMessage[200];
	int			iIndex;
	SEARCHINFO		*psi;

	switch (message) {
		case WM_INITDLG:
			hwndList = WinWindowFromID(hwnd, IDC_CODE_CODE);
			WinSendMsg(hwndList, LM_DELETEALL, 0, 0);
			WinEnableControl(hwnd, IDC_CODE_OKAY, FALSE);
			WinEnableControl(hwnd, IDC_CODE_CANCEL, FALSE);
			psi = (SEARCHINFO *)PVOIDFROMMP(mp2);
			ti.hwndCaller = hwnd;
			ti.pData = (void *)(psi->pszString);
			ti.pps = &ps;
			iIndex = -1;
			switch (psi->iType) {
				case Q_FULL:
					iIndex = _beginthread((THREADPROC)ThreadFindAllQuotes, NULL, THREADSTACK, (void *)&ti);
					break;

				case Q_KEY:
					iIndex = _beginthread((THREADPROC)ThreadFindKeywords, NULL, THREADSTACK, (void *)&ti);
					break;

				case Q_TEXT:
					iIndex = _beginthread((THREADPROC)ThreadFindQuoteText, NULL, THREADSTACK, (void *)&ti);
					break;

				case Q_NAME:
					iIndex = _beginthread((THREADPROC)ThreadFindAuthorsQuotes, NULL, THREADSTACK, (void *)&ti);
					break;
			}

			if (iIndex == -1) {
				WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				WinDismissDlg(hwnd, TRUE);
			}
			return (FALSE);

		case QM_FOUND:
			WinInsertLboxItem(hwndList, LIT_SORTASCENDING, mp1);
			return (FALSE);

		case QM_DONE:
			WinEnableControl(hwnd, IDC_CODE_OKAY, TRUE);
			WinEnableControl(hwnd, IDC_CODE_CANCEL, TRUE);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_CODE_OKAY:
					iIndex = (int)WinQueryLboxSelectedItem(hwndList);
					if (iIndex != LIT_NONE)
						WinQueryLboxItemText(hwndList, iIndex, ps.szQuoteCode, sizeof(ps.szQuoteCode));
					else {
						ps.szQuoteCode[0] = '\0';
						ps.pszText = NULL;
					}
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_CODE_CANCEL:
					ps.szQuoteCode[0] = '\0';
					ps.pszText = NULL;
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY ChooseAuthorBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the choose author box. This allows the user to select an
 * author from a list of codes.
 */
{
	static THREADINFO	ti;
	static HWND		hwndList;
	char			szMessage[200];
	int			iIndex;
	SEARCHINFO		*psi;

	switch (message) {
		case WM_INITDLG:
			hwndList = WinWindowFromID(hwnd, IDC_CODE_CODE);
			WinSendMsg(hwndList, LM_DELETEALL, 0, 0);
			WinEnableControl(hwnd, IDC_CODE_OKAY, FALSE);
			WinEnableControl(hwnd, IDC_CODE_CANCEL, FALSE);
			psi = (SEARCHINFO *)PVOIDFROMMP(mp2);
			ti.hwndCaller = hwnd;
			ti.pData = (void *)(psi->pszString);
			ti.pps = &ps;
			iIndex = -1;
			switch (psi->iType) {
				case Q_FULL:
					iIndex = _beginthread((THREADPROC)ThreadFindAllAuthors, NULL, THREADSTACK, (void *)&ti);
					break;

				case Q_TEXT:
					iIndex = _beginthread((THREADPROC)ThreadFindDescription, NULL, THREADSTACK, (void *)&ti);
					break;

				case Q_NAME:
					iIndex = _beginthread((THREADPROC)ThreadFindAuthorName, NULL, THREADSTACK, (void *)&ti);
					break;
			}
			if (iIndex == -1) {
				WinLoadString(hab, 0, IDS_THREADFAILED, sizeof(szMessage), szMessage);
				WinMessageBox(HWND_DESKTOP, hwnd, szMessage, "Error", 0, MB_ERROR);
				WinDismissDlg(hwnd, TRUE);
			}
			return (FALSE);

		case QM_FOUND:
			WinInsertLboxItem(hwndList, LIT_SORTASCENDING, mp1);
			return (FALSE);

		case QM_DONE:
			WinEnableControl(hwnd, IDC_CODE_OKAY, TRUE);
			WinEnableControl(hwnd, IDC_CODE_CANCEL, TRUE);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_CODE_OKAY:
					iIndex = (int)WinQueryLboxSelectedItem(hwndList);
					if (iIndex != LIT_NONE)
						WinQueryLboxItemText(hwndList, iIndex, ps.szAuthorCode, sizeof(ps.szAuthorCode));
					else {
						ps.szAuthorCode[0] = '\0';
						ps.pszDesc = NULL;
					}
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_CODE_CANCEL:
					ps.szAuthorCode[0] = '\0';
					ps.pszDesc = NULL;
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		case WM_CONTROL:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_CODE_CODE:
					switch (SHORT2FROMMP(mp1)) {
						case LN_ENTER:
							WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(IDC_CODE_OKAY), 0);
							return (FALSE);
					}
			}


		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY QuoteOfTheDayBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for modifying the settings used by the Quote-of-the-Day
 * program.
 */
{
	HINI		hini;
	FILEDLG		fd;
	char		szString[CCHMAXPATH];

	switch (message) {
		case WM_INITDLG:
			hini = PrfOpenProfile(hab, "QUOTER.INI");
			PrfQueryProfileString(hini, "QotD", "QDB", "", szString, sizeof(szString));
			WinSendDlgItemMsg(hwnd, IDC_QOTD_QDB, EM_SETTEXTLIMIT, MPFROMSHORT(CCHMAXPATH), 0);
			WinSetDlgItemText(hwnd, IDC_QOTD_QDB, szString);
			WinSendDlgItemMsg(hwnd, IDC_QOTD_ADB, EM_SETTEXTLIMIT, MPFROMSHORT(CCHMAXPATH), 0);
			PrfQueryProfileString(hini, "QotD", "ADB", "", szString, sizeof(szString));
			WinSetDlgItemText(hwnd, IDC_QOTD_ADB, szString);
			PrfCloseProfile(hini);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_QOTD_QDB_BROWSE:
					memset(&fd, 0, sizeof(FILEDLG));
					fd.cbSize = sizeof(FILEDLG);
					fd.fl = FDS_OPEN_DIALOG | FDS_CENTER;
					strcpy(fd.szFullFile, szQuoteExt);
					WinFileDlg(HWND_DESKTOP, hwndFrame, &fd);
					if (fd.lReturn == DID_OK)
						WinSetDlgItemText(hwnd, IDC_QOTD_QDB, fd.szFullFile);
					return (FALSE);

				case IDC_QOTD_ADB_BROWSE:
					memset(&fd, 0, sizeof(FILEDLG));
					fd.cbSize = sizeof(FILEDLG);
					fd.fl = FDS_OPEN_DIALOG | FDS_CENTER;
					strcpy(fd.szFullFile, szAuthorExt);
					WinFileDlg(HWND_DESKTOP, hwndFrame, &fd);
					if (fd.lReturn == DID_OK)
						WinSetDlgItemText(hwnd, IDC_QOTD_ADB, fd.szFullFile);
					return (FALSE);

				case IDC_QOTD_OKAY:
					hini = PrfOpenProfile(hab, "QUOTER.INI");
					WinQueryDlgItemText(hwnd, IDC_QOTD_QDB, sizeof(szString), szString);
					PrfWriteProfileString(hini, "QofD", "QDB", szString);
					WinQueryDlgItemText(hwnd, IDC_QOTD_ADB, sizeof(szString), szString);
					PrfWriteProfileString(hini, "QofD", "ADB", szString);
					PrfCloseProfile(hini);
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);

				case IDC_QOTD_CANCEL:
					WinDismissDlg(hwnd, TRUE);
					return (FALSE);
			}

		default:
			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


MRESULT EXPENTRY GetTextBox(HWND hwnd, ULONG message, MPARAM mp1, MPARAM mp2)
/*
 * Window procedure for the text entry dialogue box. Places a string entered
 * by the user into a string pointed to by the mp2 passed with the WM_INITDLG
 * message. There is a limit of 100 characters on the size of the string.
 *
 * The string is set to be empty if the box is cancelled.
 */
{
	static char	*pszString;

	switch (message) {
		case WM_INITDLG:
			pszString = (char *)PVOIDFROMMP(mp2);
			return (FALSE);

		case WM_COMMAND:
			switch (SHORT1FROMMP(mp1)) {
				case IDC_GETTEXT_OKAY:
					WinQueryDlgItemText(hwnd, IDC_GETTEXT_TEXT, 100, pszString);
					WinDismissDlg(hwnd, TRUE);
					break;

				case IDC_GETTEXT_CANCEL:
					pszString[0] = '\0';
					WinDismissDlg(hwnd, TRUE);
					break;
			}

		default:

			return (WinDefDlgProc(hwnd, message, mp1, mp2));
	}

	return (FALSE);
}


void GetINISettings(void)
/*
 * Get the relevent settings from the QUOTER.INI file.
 */
{
	HINI	hiniProfile;

	hiniProfile = PrfOpenProfile(hab, "QUOTER.INI");

	PrfQueryProfileString(hiniProfile, "Quoter", "Editor", "E.EXE", prf.szEditor, sizeof(prf.szEditor));
	PrfQueryProfileString(hiniProfile, "Quoter", "EditorParams", "%", prf.szEditorParams, sizeof(prf.szEditorParams));

	PrfCloseProfile(hiniProfile);
}


void SetINISettings(void)
/*
 * Set the relevent settings in the QUOTER.INI file.
 */
{
	HINI	hiniProfile;

	hiniProfile = PrfOpenProfile(hab, "QUOTER.INI");

	PrfWriteProfileString(hiniProfile, "Quoter", "Editor", prf.szEditor);
	PrfWriteProfileString(hiniProfile, "Quoter", "EditorParams", prf.szEditorParams);

	PrfCloseProfile(hiniProfile);
}


HAPP StartEditor(HWND hwndNotify, char *pszFileName)
/*
 * Start the editor.
 *
 * HWND hwndNotify	- handle of the window to notify when the editor terminates
 * char *pszFileName	- the file to edit
 *
 * Returns: 		- handle to the editor application on success
 *			  NULLHANDLE on failure
 */
{
	char		szParams[100];
	PROGDETAILS	pd;

	/* generate command line */
	strreplace(prf.szEditorParams, "%", pszFileName, szParams);

	/* set up application details */
	memset(&pd, 0, sizeof(PROGDETAILS));
	pd.Length = sizeof(PROGDETAILS);
	pd.progt.progc = PROG_PM;
	pd.progt.fbVisible = SHE_VISIBLE;
	pd.pszExecutable = prf.szEditor;
	pd.pszParameters = szParams;
	pd.pszStartupDir = ".";

	/* go */
	return (WinStartApp(hwndNotify, &pd, szParams, NULL, 0));
}
